#include "Engine/PhongLighting.es30.glsl"

layout(location=0) out vec4 target0;
layout(location=1) out vec4 target1;

void main()
{
vec4 screenPos = gl_FragCoord;

  VertexIn In = importIn();
  SurfaceOut surface = SURFACE_FUNCTION(In);

#ifdef COGS_ALPHA_TEST_DISCARD
  if (surface.Color.a < alphaThreshold) discard;
#endif

  SurfaceLighting surfaceLighting;
  surfaceLighting.Color = surface.Color;
  surfaceLighting.Emissive = surface.Emissive;
  surfaceLighting.Normal = surface.Normal;
  surfaceLighting.ViewSpacePosition = (1.0/In.ViewSpacePosition.w)*In.ViewSpacePosition.xyz;
  surfaceLighting.SpecularPower = surface.SpecularPower;
  surfaceLighting.Specular = surface.Specular;
  surfaceLighting.WorldPosition = In.WorldPosition;

  surfaceLighting.LightModel = GetMaterialId() | 32;

  vec3 ci = DoLighting(surfaceLighting, screenPos).rgb;
  float ai = surfaceLighting.Color.a;

  // Constants used in the equations
  float z = abs(In.ViewSpacePosition.z);
  float z_5 = z/5.0;
  float z_10 = z/10.0;
  float z_200 = z/200.0;

#if 0
  // Equation 7
  float w = 10.0/(1e-5 + pow(z_5, 2.0) + pow(z_200, 6.0));
#endif

#if 0
  // Equation 8
  float w = 10.0/(1e-5 + pow(z_10, 3.0) + pow(z_200, 6.0));
#endif

#if 1
  // Equation 9
  float w = 0.03/(1e-5 + pow(z_200, 4.0));
#endif

#if 0
  // Equation 10
  float w = 3e3 * pow(1.0-gl_FragCoord.z, 3.0);
#endif

  w = ai * clamp(w, 1e-2, 3e3);

#if 0
  // Weight used for examples.
  float w = sqrt(z + 5.0);
#endif

  // First part of Listing 4 using glBlendFuncSeparate(GL_ONE, GL_ONE, GL_ZERO, GL_ONE_MINUS_SRC_ALPHA);
  target0 = vec4(w * ci, ai);
  target1 = vec4(w * ai);
}
