#include "Engine/PhongLighting.hlsl"

struct PixelOut {
  float4 A : SV_Target0;
  float4 B : SV_Target1;
};

PixelOut main(VertexIn In, float4 screenPos : SV_Position)
{

  SurfaceOut surface = SURFACE_FUNCTION(In);

#ifdef COGS_ALPHA_TEST_DISCARD
  if (surface.Color.a < alphaThreshold) discard;
#endif

  SurfaceLighting surfaceLighting = (SurfaceLighting)0;
  surfaceLighting.Color = surface.Color;
  surfaceLighting.Emissive = surface.Emissive;
  surfaceLighting.Normal = surface.Normal;
  surfaceLighting.ViewSpacePosition = (1.0 / In.ViewSpacePosition.w) * In.ViewSpacePosition.xyz;
  surfaceLighting.SpecularPower = surface.SpecularPower;
  surfaceLighting.Specular = surface.Specular;
  surfaceLighting.WorldPosition = In.WorldPosition;

  surfaceLighting.LightModel = GetMaterialId() | 32;

  float3 ci = DoLighting(surfaceLighting, screenPos).rgb;
  float ai = surfaceLighting.Color.a;

  // Constants used in the equations
  float z = abs(In.ViewSpacePosition.z);
  float z_5 = z / 5.0;
  float z_10 = z / 10.0;
  float z_200 = z / 200.0;

#if 0
  // Equation 7
  float w = 10.0 / (1e-5 + pow(z_5, 2.0) + pow(z_200, 6.0));
#endif

#if 0
  // Equation 8
  float w = 10.0 / (1e-5 + pow(z_10, 3.0) + pow(z_200, 6.0));
#endif

#if 1
  // Equation 9
  float w = 0.03 / (1e-5 + pow(z_200, 4.0));
#endif

#if 0
  // Equation 10
  float w = 3e3 * pow(1.0 - screenPos.z, 3.0);
#endif

  w = ai * max(1e-2, min(3e3, w));

#if 0
  // Weight used for examples.
  float w = sqrt(z + 5.0);
#endif

  PixelOut Out = (PixelOut)0;
  Out.A = float4(w * ci, ai);
  Out.B = (float4)(w * ai);

  return Out;
}
