#version 300 es
precision highp float;
precision highp int;

uniform highp sampler2D colorTexture;
uniform highp usampler2D normalTexture;
uniform highp sampler2D depthTexture;

uniform mat4 inverseProjection;
uniform vec2 viewSizeRcp;

out vec4 fragColor;

vec3 worldSpaceFromScreen(vec2 screenPos, float depth)
{
  vec2 unit = viewSizeRcp * screenPos;
  vec4 h = inverseProjection * vec4(2.0 * unit - vec2(1.0), depth, 1.0);
  return (1.0 / h.w) * h.xyz;
}

void main()
{
  ivec2 P = ivec2(gl_FragCoord.xy);

  vec4 color = texelFetch(colorTexture, P, 0);

  uvec4 n00_ = texelFetch(normalTexture, P, 0);
  vec3 n00 = (1.0 / 512.0) * vec3(n00_.rgb) - vec3(1.0);
  vec3 n10 = (1.0 / 512.0) * vec3(texelFetchOffset(normalTexture, P, 0, ivec2(1, 0)).rgb) - vec3(1.0);
  vec3 n01 = (1.0 / 512.0) * vec3(texelFetchOffset(normalTexture, P, 0, ivec2(0, 1)).rgb) - vec3(1.0);


  float intensity = 1.0;
  if (0.1 < dot(n00, n00) || 0.1 < dot(n10, n10) || 0.1 < dot(n01, n01))
  {
    float d_00 = texelFetch(depthTexture, P, 0).r;
    float d_10 = texelFetchOffset(depthTexture, P, 0, ivec2(1, 0)).r;
    float d_01 = texelFetchOffset(depthTexture, P, 0, ivec2(0, 1)).r;

    vec3 wpos_00 = worldSpaceFromScreen(gl_FragCoord.xy + vec2(0, 0), d_00);
    vec3 wpos_10 = worldSpaceFromScreen(gl_FragCoord.xy + vec2(1, 0), d_10);
    vec3 wpos_01 = worldSpaceFromScreen(gl_FragCoord.xy + vec2(0, 1), d_01);

    // Distance from tangent plane
    vec3 dir_10 = normalize(wpos_10 - wpos_00);
    vec3 dir_01 = normalize(wpos_01 - wpos_00);
    float g = max(max(dot(n00, dir_01),
                      dot(n01, dir_01)),
                  max(dot(n00, dir_10),
                      dot(n10, dir_10)));

    // Difference in normals
    float a = min(dot(n00, n10),
                  dot(n00, n01));

    if(0.5 < g || a < 0.95) {
      intensity = 0.25;
    }
  }

  // Output is onscreen, convert to srgb
  color.rgb = clamp(intensity * color.rgb, vec3(0.0), vec3(1.0));
  color.rgb = mix(pow(color.rgb, vec3(1.0/2.4)),
                  12.92 * color.rgb,
                  lessThan(color.rgb, vec3(0.0031308)));

  fragColor = color;
}
