#if defined(COGS_DEBUGVIEW_OBJECTID)
#include "Random.es30.glsl"
#endif

#if defined(COGS_DEBUGVIEW_SHADOW_CASCADES)
#include "Engine/PhongLighting.es30.glsl"
#endif

in vec4 vs_fs_A;
in vec4 vs_fs_B;
in vec4 vs_fs_C;

layout(location=0) out vec4 target0;
layout(location=1) out uvec4 target1;

void main()
{
  VertexIn In = importIn();
  SurfaceOut surface = SURFACE_FUNCTION(In);

  vec4 A = 0.5 * fwidth(vs_fs_A);
  vec4 B = 0.5 * fwidth(vs_fs_B);
  vec4 C = 0.5 * fwidth(vs_fs_C);

#if defined(COGS_ALPHA_TEST_DISCARD) || defined(COGS_ALPHA_TO_COVERAGE)
  if (surface.Color.a < alphaThreshold) discard;
#endif

  float na = dot(vec4(greaterThan(A, vec4(0.001))), vec4(1.0));
  float nb = dot(vec4(greaterThan(B, vec4(0.001))), vec4(1.0));
  float nc = dot(vec4(greaterThan(C, vec4(0.001))), vec4(1.0));

  float e[12] = float[12](
    vs_fs_A.x, vs_fs_A.y, vs_fs_A.z, vs_fs_A.w,
    vs_fs_B.x, vs_fs_B.y, vs_fs_B.z, vs_fs_B.w,
    vs_fs_C.x, vs_fs_C.y, vs_fs_C.z, vs_fs_C.w
  );

  float d[12] = float[12](
    A.x, A.y, A.z, A.w,
    B.x, B.y, B.z, B.w,
    C.x, C.y, C.z, C.w
  );

  float max_ = 0.0;
  for (int i = 0; i < 12; i++) {
    max_ = max(max_, d[i]);
  }

  float dist = 1.0;
  float min_ = max_;
  for (int j = 0; j < 12; j++) {
    if (0.001 < d[j]) {
      min_ = min(min_, d[j]);
      dist = min(dist, (1.0/d[j])* min(1.0 - e[j], e[j]));
    }
  }

#if defined(COGS_DEBUGVIEW_TRIANGLESIZE)

  vec4 D = (A + B + C);
  float a = 0.5 * (min_ + max_);
  float s = 1.0 / a;
  target0 = vec4(s);

#elif defined(COGS_DEBUGVIEW_OBJECTID)

  uint id = wangHash(objectId);
  target0 = vec4((1.0 / 255.0)*float((id >>  0u) & 255u),
                   (1.0 / 255.0)*float((id >>  8u) & 255u),
                   (1.0 / 255.0)*float((id >> 16u) & 255u),
                   1.0);

#elif defined(COGS_DEBUGVIEW_NORMAL)

  target0 = vec4(0.5 * (surface.Normal + vec3(1)), 1.0);

#elif defined(COGS_DEBUGVIEW_SHADOW_CASCADES)
  if ((GetMaterialId() & COGS_LIGHT_MASK_PHONG) == COGS_LIGHT_MASK_PHONG) {
    vec3 color = vec3(1,1,1);
    float shadowFactor = 1.0;

#ifdef COGS_FORWARD_SHADOWS_ENABLED
    for (int i = 0; i < numLights.x; ++i) {

      ShadowData shadow = shadows[i];
      if((shadow.enabled != 0) && (lightPositions[i].w == 0.0)) {
        float viewZ = dot(shadow.cascadeLine.xyz, In.WorldPosition.xyz) + shadow.cascadeLine.w;

        if (viewZ <= shadow.farSplits[shadow.numCascades - 1]) {
          int index = 0;
          for (int c = shadow.numCascades - 1; c >= 0; --c) {
            if (viewZ < shadow.farSplits[c]) {
              color = vec3(((c+1)&1) != 0 ? 1.0 : 0.0,
                           ((c+1)&2) != 0 ? 1.0 : 0.0,
                           ((c+1)&4) != 0 ? 1.0 : 0.0);
              index = c;
            }
          }
        }

        vec4 lightPosition = lightPositions[i];
        shadowFactor = calculateShadowContribution(In.WorldPosition, surface.Normal, lightDirections[i].xyz, i, gl_FragCoord);
        break;
      }
    }
#endif

    target0 = vec4(shadowFactor*color, 1.0);
  }
  else {
    target0 = vec4(0.2, 0.2, 0.2, 1.0);
  }
#endif

  vec3 n = 512.0 * clamp(vec3(1.0) + mat3(viewMatrix) * surface.Normal, vec3(0.0), vec3(2.0));
  if ((GetMaterialId() & COGS_LIGHT_MASK_PHONG) != COGS_LIGHT_MASK_PHONG) {
    n = vec3(512.0);
  }
  target1 = uvec4(n.x, n.y, n.z, uint(max(0.0, 2.0 * dist)));
}
