#version 300 es
precision highp float;
precision highp int;

#if defined(COGS_DEBUGVIEW_HAS_COLOR)
uniform highp sampler2D colorTexture;
#else
uniform highp sampler2D valueTexture;
#endif
uniform highp usampler2D normalTexture;
uniform highp sampler2D depthTexture;

uniform float mid;
uniform float high;
uniform float edge;
uniform float outline;

uniform mat4 inverseProjection;
uniform vec2 viewSizeRcp;

out vec4 fragColor;

vec3 worldSpaceFromScreen(vec2 screenPos, float depth)
{
  vec2 unit = viewSizeRcp * screenPos;
  vec4 h = inverseProjection * vec4(2.0 * unit - vec2(1.0), depth, 1.0);
  return (1.0 / h.w) * h.xyz;
}

void main()
{
  ivec2 P = ivec2(gl_FragCoord.xy);

  uvec4 n00_ = texelFetch(normalTexture, P, 0);
  vec3 n00 = (1.0 / 512.0) * vec3(n00_.rgb) - vec3(1.0);
  vec3 n10 = (1.0 / 512.0) * vec3(texelFetchOffset(normalTexture, P, 0, ivec2(1, 0)).rgb) - vec3(1.0);
  vec3 n01 = (1.0 / 512.0) * vec3(texelFetchOffset(normalTexture, P, 0, ivec2(0, 1)).rgb) - vec3(1.0);

  float a = min(dot(n00, n10),
                dot(n00, n01));

#if defined(COGS_DEBUGVIEW_HAS_COLOR)

  vec3 color = texelFetch(colorTexture, P, 0).rgb;

#else

  vec3 color = vec3(0, 0.2, 0.3);
  float value = texelFetch(valueTexture, P, 0).r;
  if (0.0 < value) {
    if(value < mid) {
      float t = value / mid;
      color = vec3(1, t, 0);
    }
    else if (value < high) {
      float t = (value - mid) / (high-mid);
      color = vec3((1.0 - t), 1, 0);
    }
    else {
      color = vec3(0, 1, 0);
    }
  }

#endif

  color = mix(color, vec3(0.0), clamp(edge * (1.0 - 0.5 * float(n00_.w)), 0.0, 1.0));

  if (0.1 < dot(n00, n00) || 0.1 < dot(n10, n10) || 0.1 < dot(n01, n01))
  {
    float d_00 = texelFetch(depthTexture, P, 0).r;
    float d_10 = texelFetchOffset(depthTexture, P, 0, ivec2(1, 0)).r;
    float d_01 = texelFetchOffset(depthTexture, P, 0, ivec2(0, 1)).r;

    vec3 wpos_00 = worldSpaceFromScreen(gl_FragCoord.xy + vec2(0, 0), d_00);
    vec3 wpos_10 = worldSpaceFromScreen(gl_FragCoord.xy + vec2(1, 0), d_10);
    vec3 wpos_01 = worldSpaceFromScreen(gl_FragCoord.xy + vec2(0, 1), d_01);

    // Distance from tangent plane
    vec3 dir_10 = normalize(wpos_10 - wpos_00);
    vec3 dir_01 = normalize(wpos_01 - wpos_00);
    float g = max(max(dot(n00, dir_01),
                      dot(n01, dir_01)),
                  max(dot(n00, dir_10),
                      dot(n10, dir_10)));

    // Difference in normals
    float a = min(dot(n00, n10),
                  dot(n00, n01));

    if(0.5 < g || a < 0.95) {
      color = mix(color, vec3(1.0), outline);
    }
  }

  fragColor = vec4(color, 1);
}
