#include "AdaptivePlanarGrid.hlsl"

#ifdef COGS_BASIC_OCEAN_QUUX
#include "BasicOceanEffectQuux.hlsl"
#else
#include "BasicOceanEffect.hlsl"
#endif

VertexOut VertexFunction(VertexIn In)
{
  // Snap edge vertices to neighbouring grids
  float2 pos = snapAdaptivePlanarGridEdges(In.Position.xyz).xy;

  // Move and scale patch to its position in world space
  float3 world = mul(float4(pos, 0, 1),worldMatrix).xyz;

  // Calculate coordinates in wave space.
  float2 waveCoordinates = getTexCoordWorld(pos.xy);
  
  // And we only displace if we are relatively close to the camera
  float3 surfacePositionWorld = world;
  float4 surfacePositionView = mul(float4(surfacePositionWorld, 1.0), viewMatrix);

  // Determine pixel size of significant waveheight, which we use to control LoD
#if COGS_BASIC_OCEAN_QUUX
  float4 a0 = mul(float4(world, 1), worldToClipMatrix);
  float4 a1 = mul(float4(world + significantWaveHeight * cameraYAxis.xyz, 1), worldToClipMatrix);
  float2 h = viewportScale * ((1.0 / a0.w)*a0.xy - (1.0 / a1.w)*a1.xy);
  float lod = length(h);

  surfacePositionWorld += waveDisplace(world.xy, lod);

#else
  float4 a0 = mul(float4(world, 1.0), worldToClipMatrix);
  float4 a1 = mul(float4(world + cameraYAxis.xyz, 1.0), worldToClipMatrix);
  float lod = significantWaveHeight * cameraYAxis.w * ((a1.y / a1.w) - (a0.y / a0.w));
  if (lod > 2.0) {
    float blend = clamp(lod - 2.0, 0.0, 1.0);
    surfacePositionWorld += blend * waveDisplace(waveCoordinates);
  }
#endif

  surfacePositionWorld.z += seaLevel;

  // Output results
  VertexOut Out = (VertexOut)0;
  Out.undisplacedPositionWorld = world.xy;

#if COGS_BASIC_OCEAN_CURVED
  // Assume TransformSystem's origin is tangent on earth
  float R = 6371000.0;
  //R *= 1.0 / 200;
  float d = length(world.xy);
  surfacePositionWorld.z -= R - sqrt((R - d)*(R + d));
#endif
    

  Out.Position = mul(float4(surfacePositionWorld, 1.0), worldToClipMatrix);
  Out.WorldPosition = float4(surfacePositionWorld, 1);

  Out.screenPosition = Out.Position;
  Out.wavePosition = float3(waveCoordinates, lod);

  return Out;
}
