
#if defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED)
float elevation(vec2 texpos)
{
  return textureLod(Elevation, texpos, 0.0).r;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_ALPHA)
float elevation(vec2 texpos)
{
  return textureLod(Elevation, texpos, 0.0).a;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED_GREEN_BLUE)
float elevation(vec2 texpos)
{
  vec3 value = textureLod(Elevation, texpos, 0.0).rgb;
  return 256.0 * 256.0 * 256.0 * value.r + 256.0 * 256.0 * value.g + 256.0 * value.b;
}
#endif

vec3 vibranceFilter(vec3 color)
{
  if (vibrance == 1.0) return color;

  float ma = max(color.r, max(color.g, color.b));
  float mi = min(color.r, min(color.g, color.b));
  float saturation = ma - mi;
  float scale = pow(abs(saturation), abs(vibrance)) / max(saturation, 0.01);

  vec3 white = vec3(1.0 / sqrt(3.0));
  vec3 grey = dot(color, white) * white;

  return grey + scale * (color - grey);
}

SurfaceOut surfaceFunction(VertexIn vertex)
{
  SurfaceOut Out = createSurfaceOut();

#ifdef COGS_BASIC_TERRAIN_HAS_NODATA
  if (vertex.texCoord.z == noDataValue) discard;
#endif

#if defined(COGS_BASIC_TERRAIN_SOLID)
  vec4 c = diffuseColor;
#elif defined(COGS_BASIC_TERRAIN_TEXTURE_RGB)
  vec4 c = vec4(texture(Color, vertex.texCoord.xy).rgb, 1);
  c.rgb = diffuseColor.rgb * vibranceFilter(c.rgb);
#elif defined(COGS_BASIC_TERRAIN_TEXTURE_RGBA)
  vec4 c = texture(Color, vertex.texCoord.xy);
  c.rgb = diffuseColor.rgb * vibranceFilter(c.rgb);
#endif

#ifdef COGS_BASIC_TERRAIN_ELEVATION_TO_ALPHA
  {
    float e = elevation(vertex.texCoord.xy);
    vec3 ctr = elevationToAlphaTransform;
    c.a = pow(saturate(ctr.y * (e - ctr.x)), ctr.z);
  }
#endif

  float dh_du = elevation(vertex.texCoord.xy + vec2(elevationTransform.z, 0))
              - elevation(vertex.texCoord.xy - vec2(elevationTransform.z, 0));
  float dh_dv = elevation(vertex.texCoord.xy + vec2(0, elevationTransform.w))
              - elevation(vertex.texCoord.xy - vec2(0, elevationTransform.w));

  Out.Normal = normalize(cross(vec3(normalTransform.x, 0, dh_du),
                               vec3(0, normalTransform.y, dh_dv)));
  Out.Color = vec4(c.rgb, c.a);
  Out.Specular = vec3(0.0, 0.0, 0.0);
  Out.SpecularPower = 0.0;

  return Out;
}
