#include "Default/DefaultCommon.es20.glsl"

VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = createVertexOut();

  vec4 inPosition = positionTypeAsFloat4(In.Position);

#ifdef COGS_VARIANT_SKINNED_BASIC
  const float w0 = In.BoneWeights.x;
  const int i0 = In.BoneIndexes.x;

  const float w1 = In.BoneWeights.y;
  const int i1 = In.BoneIndexes.y;

  const float w2 = In.BoneWeights.z;
  const int i2 = In.BoneIndexes.z;

  const float w3 = In.BoneWeights.w;
  const int i3 = In.BoneIndexes.w;

  vec4 position = w0 * boneTransforms[i0] * inPosition;
  position     += w1 * boneTransforms[i1] * inPosition;
  position     += w2 * boneTransforms[i2] * inPosition;
  position     += w3 * boneTransforms[i3] * inPosition;

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  vec3 normal = w0 * mat3(boneTransforms[i0]) * In.Normal;
  normal     += w1 * mat3(boneTransforms[i1]) * In.Normal;
  normal     += w2 * mat3(boneTransforms[i2]) * In.Normal;
  normal     += w3 * mat3(boneTransforms[i3]) * In.Normal;

#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  vec3 tangent = w0 * mat3(boneTransforms[i0]) * In.Tangent;
  tangent     += w1 * mat3(boneTransforms[i1]) * In.Tangent;
  tangent     += w2 * mat3(boneTransforms[i2]) * In.Tangent;
  tangent     += w3 * mat3(boneTransforms[i3]) * In.Tangent;
#endif
#endif

#else
  vec4 position = inPosition;

#ifdef DEFAULT_MATERIAL_INSTANCED_POSITIONS
  position.xyz += In.InstancePosition.xyz;
#endif
#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)

#if BASE_MATERIAL_VERTEX_NORMAL0 == COGS_FLOAT2
  vec3 normal = octDecode(In.Normal);
#else
  vec3 normal = In.Normal;
#endif

#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  vec3 tangent = In.Tangent;
#endif
#endif
#endif

  Out.WorldPosition = worldMatrix * position;

#ifdef DEFAULT_MATERIAL_INSTANCED_WORLD_POSITIONS
  Out.WorldPosition.xyz += In.InstanceWorldPosition.xyz;
#endif
  Out.Position = worldToClipMatrix * Out.WorldPosition;

#if defined(DEFAULT_MATERIAL_ENABLE_LIGHTING) && defined(BASE_MATERIAL_VERTEX_NORMAL0)
  Out.Normal = mat3(worldMatrix) * normal;

#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  Out.Tangent = mat3(worldMatrix) * tangent;
#endif

#endif

#ifdef COGS_DEFAULT_MATERIAL_TEXTURED
  Out.TexCoord = In.TexCoord;
#endif

#ifdef DEFAULT_MATERIAL_INSTANCED_COLORS
  Out.InstanceColor = In.InstanceColor;
#endif

  return Out;
}
