#ifndef COGS_COMMON_ES30_GLSL
#define COGS_COMMON_ES30_GLSL

#define COGS_SCENEFLAGS_NONE                  (    0u)
#define COGS_SCENEFLAGS_OUTPUT_SRGB           (1u<<0u)
#define COGS_SCENEFLAGS_SUBMERGED             (1u<<1u)

#ifdef COGS_SCENEBUFFER_REFERENCED
uniform SceneBuffer
{
  mat4 projectionMatrix;
  mat4 viewMatrix;
  mat4 inverseViewMatrix;
  mat4 inverseProjectionMatrix;
  mat4 worldToClipMatrix;
  vec4 projectionParameters;
  vec2 viewportOrigin;
  vec2 viewportSize;
  vec4 clippingPlanes[6];
  uvec4 blueNoiseOffset;
  float shadowDepthClamp;
  float animationTime;
  float exposure;
  uint sceneFlags;
};
#endif

#ifdef COGS_OBJECTBUFFER_REFERENCED
uniform ObjectBuffer
{
  mat4 worldMatrix;
  uint objectId;
};
#endif

#ifdef COGS_ANIMATIONBUFFER_REFERENCED
uniform AnimationBuffer
{
  mat4 boneTransforms[256];
};
#endif

#ifdef COGS_ENVIRONMENT_SKY_REFERENCED
uniform samplerCube environmentSky;
#endif

#ifdef COGS_ENVIRONMENT_RADIANCE_REFERENCED
uniform samplerCube environmentRadiance;
#endif

#ifdef COGS_ENVIRONMENT_IRRADIANCE_REFERENCED
uniform samplerCube environmentIrradiance;
#endif

#ifdef COGS_AMBIENT_IRRADIANCE_REFERENCED
uniform samplerCube ambientIrradiance;
#endif

#ifdef COGS_CASCADEDSHADOWMAP_REFERENCED
#ifdef COGS_CASCADEDSHADOWMAP_HIGH_PRECISION
uniform sampler2DArrayShadow cascadedShadowMap;
#else
uniform lowp sampler2DArrayShadow cascadedShadowMap;
#endif
#endif

#ifdef COGS_CUBESHADOWMAP_REFERENCED
#ifdef COGS_CUBESHADOWMAP_HIGH_PRECISION
uniform samplerCubeShadow cubeShadowMap;
#else
uniform lowp samplerCubeShadow cubeShadowMap;
#endif
#endif


struct ShadowData
{
  vec4 cascadeLine;
  vec4 nearSplits;
  vec4 farSplits;
  mat4 lightMatrix[6];
  float shadowSampleBias;
  int numCascades;
  int enabled;
  float texelSize;
};

#ifdef COGS_SHADOWBUFFER_REFERENCED
uniform ShadowBuffer
{
  ShadowData shadows[COGS_MAX_LIGHTS];
  ivec4 cascadeOffsets[COGS_MAX_LIGHTS];
};
#endif

#ifdef COGS_LIGHTBUFFER_REFERENCED
uniform LightBuffer
{
  vec4 lightPositions[COGS_MAX_LIGHTS];
  vec4 lightDirections[COGS_MAX_LIGHTS];
  vec4 lightColorIntensity[COGS_MAX_LIGHTS];
  vec4 lightParameters[COGS_MAX_LIGHTS];
  ivec4 numLights;
  vec4 eyePosition;
  vec4 fogColor;
  float fogDistance;
  float fogAmount;
  int fogEnabled;
  float ambientIntensity;
  vec3 ambientColor;
  float environmentBrightness;
  float skyMultiplier;
  int seaFlags;
  int flags;  // 1: has skydome, 2: has radiance, 3: has irradiance, 4: has subsea radiance.
};
#endif

// FIXME
uniform vec4 environmentInfo;
float environmentRadianceMips() { return environmentInfo.x; }
float environmentIrradianceMips() { return environmentInfo.y; }


struct SurfaceOut
{
  vec4 Color;
  vec3 Specular;
  float SpecularPower;
  vec3 Emissive;
  vec3 Normal;

#ifdef COGS_USE_OBJECTID
  uint objectId;
#endif

#ifdef COGS_CUSTOM_DEPTH_WRITE
  float Depth;
#endif
};

SurfaceOut createSurfaceOut()
{
  SurfaceOut t;
  t.Color = vec4(0, 0, 0, 1);
  t.Specular = vec3(0);
  t.SpecularPower = 0.0;
  t.Emissive = vec3(0);
  t.Normal = vec3(0);

#ifdef COGS_USE_OBJECTID
  t.objectId = objectId;
#endif

#ifdef COGS_CUSTOM_DEPTH_WRITE
  t.Depth = 0;
#endif
 
 return t;
}

#define COGS_LIGHT_MASK_PHONG 1
#define COGS_LIGHT_MASK_INDIRECT 64
#define COGS_LIGHT_MASK_SHADOW_RECEIVER 128

#if BASE_MATERIAL_VERTEX_POSITION0 == COGS_FLOAT2
vec4 positionTypeAsFloat4(vec2 p) { return vec4(p, 0.0, 1.0); }
#elif BASE_MATERIAL_VERTEX_POSITION0 == COGS_FLOAT3
vec4 positionTypeAsFloat4(vec3 p) { return vec4(p, 1.0); }
#else
vec4 positionTypeAsFloat4(vec4 p) { return p; }
#endif

vec3 octDecode(vec2 f)
{
  // From https://twitter.com/Stubbesaurus/status/937994790553227264
  vec3 n = vec3(f.x, f.y, 1.0 - abs(f.x) - abs(f.y));
  float t = max(-n.z, 0.0);
  n.x += 0.0 <= n.x ? -t : t;
  n.y += 0.0 <= n.y ? -t : t;
  return normalize(n);
}

int GetMaterialId()
{
  int materialId = 0;

#if defined(COGS_LIGHT_MODEL_PHONG)
  materialId |= COGS_LIGHT_MASK_PHONG;
#elif defined(COGS_LIGHT_MODEL_STANDARD)
  materialId |= 4;
#endif

#ifdef COGS_SURFACE_CLOSED
  materialId |= 16;
#endif

#ifdef COGS_INDIRECT_LIGHTING_FULL
  materialId |= COGS_LIGHT_MASK_INDIRECT;
#endif

#ifdef COGS_SHADOW_RECEIVER 
  materialId |= COGS_LIGHT_MASK_SHADOW_RECEIVER;
#endif

  return materialId;
}

#endif
