float4 ShowNormals(SurfaceOut surface)
{
  return float4(float3(0.5, 0.5, 0.5) + surface.Normal * 0.5, 1.0);
}

float4 ShowColor(SurfaceOut surface)
{
  return surface.Color;
}

#include "Engine/PhongLighting.hlsl"

#ifdef COGS_DITHER_FORWARD
#include "BlueNoise.hlsl"
#endif

struct PsOutput
{
  float4 Color : SV_Target0;
#ifdef COGS_OUTPUT_NORMALS
  float3 Normal : SV_Target1;
#endif
#ifdef COGS_CUSTOM_DEPTH_WRITE
  float Depth : SV_Depth;
#endif
};

PsOutput main(float4 screenPos : SV_Position, VertexIn In)
{
  PsOutput Out = (PsOutput)0;

  SurfaceOut surface = SURFACE_FUNCTION(In);

#ifdef COGS_ALPHA_TEST_DISCARD
  if (surface.Color.a < alphaThreshold) discard;
#endif

  SurfaceLighting surfaceLighting = (SurfaceLighting)0;
  surfaceLighting.Color = surface.Color;
  surfaceLighting.Emissive = surface.Emissive;
  surfaceLighting.Normal = surface.Normal;
  surfaceLighting.ViewSpacePosition = (1.0/In.ViewSpacePosition.w)*In.ViewSpacePosition.xyz;
  surfaceLighting.SpecularPower = surface.SpecularPower;
  surfaceLighting.Specular = surface.Specular;
  surfaceLighting.WorldPosition = In.WorldPosition;

  float4 color;

#ifdef COGS_DEBUG_VIEW_NORMALS
  color = float4(surface.Normal * 0.5 + 0.5, 1);
#elif defined(COGS_LIGHT_MODEL_NONE)
  color = float4(1, 1, 1, 1);
#else
  surfaceLighting.LightModel = GetMaterialId() | 32;
  float3 light = DoLighting(surfaceLighting, screenPos).rgb;
  color = float4(light, surfaceLighting.Color.a);
#endif

#ifdef COGS_DITHER_FORWARD
  float3 dither = (blueNoise3(int2(screenPos.xy))*2.0-1.0)*(1.0/255);
  color.rgb = saturate(color.rgb + dither);
#endif

  Out.Color = color;

#ifdef COGS_OUTPUT_NORMALS
  Out.Normal = surface.Normal;
#endif
#ifdef COGS_CUSTOM_DEPTH_WRITE
  Out.Depth = surface.Depth;
#endif

  return Out;
}
