
float vectorToDepth(in vec3 v, in float n, in float f)
{
  vec3 absVector = abs(v);
  float localViewZ = max(absVector.x, max(absVector.y, absVector.z));

  float normalizedZ = (f + n) / (f - n) + ((-2.0 * f * n) / (f - n)) * (1.0 / localViewZ);

#ifdef COGS_REVERSE_DEPTH
  normalizedZ = -normalizedZ;
#endif

  return normalizedZ*0.5 + 0.5;
}

float calculatePointLightShadow(in vec4 worldLightPos, in vec3 worldPosition, in vec3 normal, in int index)
{
  ShadowData shadow = shadows[index];

  if (shadow.enabled == 0) return 1.0;

  vec3 toLight = worldLightPos.xyz - worldPosition;
  vec3 lightDir = normalize(toLight);

  //vec3 worldSpaceNormal = normal;
  //float normalSlopeScale = saturate(1 - dot(worldSpaceNormal, lightDir));

  //worldPosition.xyz += normalSlopeScale * worldSpaceNormal * shadow.normalShadowBias;

  toLight = worldLightPos.xyz - worldPosition;
  lightDir = normalize(toLight);

  float depth = vectorToDepth(toLight, shadow.nearSplits[0], shadow.farSplits[0]);
  depth = clamp(depth - shadow.shadowSampleBias, 0.0, 1.0);

  float shadowFactor = 0.0;

#if defined(COGS_SOFT_SHADOWS_HIGH)
  #define DISKSAMPLES 20
#elif defined(COGS_SOFT_SHADOWS_LOW)
  #define DISKSAMPLES 8
#else
  #define DISKSAMPLES 1
#endif

#if DISKSAMPLES > 1
  const vec3 samplingDisk[20] = vec3[20](
    vec3( 1, 1, 1 ), vec3(  1, -1, 1 ), vec3( -1, -1,  1 ), vec3( -1, 1,  1 ),
    vec3( 1, 1,-1 ), vec3(  1, -1,-1 ), vec3( -1, -1, -1 ), vec3( -1, 1, -1 ),
    vec3( 1, 1, 0 ), vec3(  1, -1, 0 ), vec3( -1, -1,  0 ), vec3( -1, 1,  0 ),
    vec3( 1, 0, 1 ), vec3( -1,  0, 1 ), vec3(  1,  0, -1 ), vec3( -1, 0, -1 ),
    vec3( 0, 1, 1 ), vec3(  0, -1, 1 ), vec3(  0, -1, -1 ), vec3(  0, 1, -1 )
  );
  for (int j = 0; j < DISKSAMPLES; j++) {
    vec3 shadowCoords = -lightDir + samplingDisk[j] * shadow.texelSize;

    shadowFactor += texture(cubeShadowMap, vec4(shadowCoords.xzy, depth));
  }

  shadowFactor *= (1.0 / float(DISKSAMPLES));
#else

  shadowFactor = texture(cubeShadowMap, vec4(-lightDir.xzy, depth));

#endif

  return shadowFactor;
}
