#include "BasicOceanEffect.es20.glsl"

uniform sampler2D SkyDome;

const float eta = 0.66;
const float FresnelPower = 2.0;
const float F = ((1.0 - eta)*(1.0 - eta)) / ((1.0 + eta)*(1.0 + eta));

float longitude(vec2 d)
{
  vec2 planeDir = normalize(-d);
  float camRelAzimuth = (0.5 / 3.1416)*acos(min(1.0, dot(planeDir, camPlaneDir)));
  if (planeDir.y*camPlaneDir.x - planeDir.x*camPlaneDir.y > 0.0) {
    camRelAzimuth = -camRelAzimuth;
  }
  return camRelAzimuth + camAzimuth;
}

float fresnel(vec3 view, vec3 normal)
{
  float refractionIndexRatio = 1.33;
  float R0 = pow(1.0 - refractionIndexRatio, 2.0) / pow(1.0 + refractionIndexRatio, 2.0);
  float cosAngle = 1.0 - clamp(dot(view, normal), 0.0, 1.0);

  float result = cosAngle * cosAngle;
  result = result * result;
  result = result * cosAngle;
  result = clamp((result * 1.0 - clamp(R0, 0.0, 1.0)) + R0, 0.0, 1.0);

  return result;
}

void main()
{
  vec3 n = waveNormal(vertex_wavePosition) * vec3(2.0, 2.0, 1.0);
  vec3 v = normalize(inverseViewMatrix[3].xyz - vertex_displacedPositionWorld);

  float fresnelTerm = fresnel(v, n);

  if (!gl_FrontFacing) {
    vec3 r = refract(v, n, 1.33);

    if (r.z < 0.02) {
      // refraction into air
      gl_FragColor = texture2D(SkyDome, vec2(longitude(r.xy), r.z * r.z));

      return;
    }
  }

  vec2 screenPosition = vec2(0.5) * (vertex_screenPosition.xy / vertex_screenPosition.w) + vec2(0.5);
  
  vec4 reflectedColor = texture2D(PlanarReflection, screenPosition + (n.xy * 0.05));

  gl_FragColor = vec4(mix(waterColor, reflectedColor, fresnelTerm * 0.8).rgb, mix(waterColor.a, 1.0, fresnelTerm));
}
