#include "Engine/Common.hlsl"
#include "Engine/PhongLighting.hlsl"

Texture2D colorTexture;
Texture2D emissiveTexture;
Texture2D specularTexture;
Texture2D normalTexture;
Texture2D depthTexture;

texture2D indirectLightTexture;

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

struct PixelOut
{
  float4 Color : SV_Target;
  float Depth : SV_Depth;
};

PixelOut main(VertexIn In, float4 ScreenPos : SV_Position)
{
  const float4 color = colorTexture.Load(int3(ScreenPos.xy, 0));
  const float4 emissive = emissiveTexture.Load(int3(ScreenPos.xy, 0));
  const float4 specular = specularTexture.Load(int3(ScreenPos.xy, 0));
  const float4 normal = normalTexture.Load(int3(ScreenPos.xy, 0));
  const float depth = depthTexture.Load(int3(ScreenPos.xy, 0)).r;

  const float4 indirectLight = indirectLightTexture.Load(int3(ScreenPos.xy, 0));

  if (color.a == 0) discard;

  float4 viewSpacePosition = mul(float4(In.ScreenPosition.xy, depth, 1), inverseProjectionMatrix);
  viewSpacePosition /= viewSpacePosition.w;

  float4 worldPosition = mul(viewSpacePosition, inverseViewMatrix);
  worldPosition.xyz *= (1.0 / worldPosition.w);

  SurfaceLighting surface = (SurfaceLighting)0;
  surface.Color = color;
  surface.Normal = normal.xyz;
  surface.Specular = specular.xyz;
  surface.ViewSpacePosition = viewSpacePosition.xyz;
  surface.SpecularPower = normal.w;
  surface.Emissive = emissive.rgb;
  surface.LightModel = (int)(255 * specular.w);
  surface.WorldPosition = worldPosition;

  PixelOut Out = (PixelOut)0;
  Out.Color = float4(DoLighting(surface, ScreenPos, indirectLight).rgb, 1.0);
  Out.Depth = depth;

  return Out;
}
