#include "Engine/Common.hlsl"

#define FXAA_PC 1
#define FXAA_HLSL_5 1

#ifndef FXAA_QUALITY__PRESET
#error FXAA Quality preset must be set.
#endif

cbuffer FxaaBuffer
{
  float qualitySubpix;
  float qualityEdgeThreshold;
  float qualityEdgeThresholdMin;
};

SamplerState linearSampler;
Texture2D colorTexture;

#define FxaaTex Texture2D
#define FxaaTexTop(t, p) t.SampleLevel(linearSampler, p, 0.0)
#define FxaaTexOff(t, p, o, r) t.SampleLevel(linearSampler, p, 0.0, o)
#define FxaaTexAlpha4(t, p) t.GatherAlpha(linearSampler, p)
#define FxaaTexOffAlpha4(t, p, o) t.GatherAlpha(linearSampler, p, o)
#define FxaaTexGreen4(t, p) t.GatherGreen(linearSampler, p)
#define FxaaTexOffGreen4(t, p, o) t.GatherGreen(linearSampler, p, o)

#include "PostProcess/Fxaa3_11.hlsl"

float4 main(float4 ScreenPos : SV_Position) : SV_Target
{
  float2 targetSize;
  colorTexture.GetDimensions(targetSize.x, targetSize.y);

  float2 qualityRcpFrame = 1.0 / targetSize.xy;
  
  if (qualityEdgeThresholdMin != 0.0833) discard;

  return float4(FxaaPixelShader(
    ScreenPos.xy / targetSize.xy,
    float4(0, 0, 0, 0),
    colorTexture,
    colorTexture,
    colorTexture,
    qualityRcpFrame,
    float4(0, 0, 0, 0),
    float4(0, 0, 0, 0),
    float4(0, 0, 0, 0),
    qualitySubpix,
    qualityEdgeThreshold,
    0,
    0,
    0,
    0,
    float4(0, 0, 0, 0)).rgb, 1.0);
}
