cbuffer SceneParameters
{
  float4x4 projectionMatrix;
  float4x4 viewMatrix;
  float4x4 inverseViewMatrix;
  float4x4 inverseProjectionMatrix;
};

cbuffer Parameters
{
  float4 color;
  float brightness;
};

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

Texture2D colorTexture;
Texture2D depthTexture;

float4 main(VertexIn In, float4 ScreenPos : SV_Position) : SV_Target
{
  const float depth = depthTexture.Load(int3(ScreenPos.xy, 0)).r;
  const float4 colorIn = colorTexture.Load(int3(ScreenPos.xy, 0));

  float4 viewSpacePosition = mul(float4(In.ScreenPosition.xy, depth, 1), inverseProjectionMatrix);
  viewSpacePosition /= viewSpacePosition.w;
  float4 worldPosition = mul(viewSpacePosition, inverseViewMatrix);

  return float4(worldPosition.xyz, 1);
}
