#ifdef COGS_REVERSE_DEPTH
#define COGS_CLEAR_DEPTH 0.0
#else
#define COGS_CLEAR_DEPTH 1.0
#endif

float2 TransformImage(in float2 offset, in float aspect)
{
  float4 pos = float4(offset.x * aspect, offset.y, 0, 1);

  pos = mul(imageTransform, pos);

  return float2(pos.x / aspect, pos.y);
}

void quadExpand(out float4 position,
                out float4 weights,
                out float2 texCoords,
                in float4 inPosition,
                in float4 inParameters,
                in float4 inTexCoords,
                in float index)
{
  float3 p = inPosition.xyz / inPosition.w;
  float2 size = inParameters.xy;
  float2 offset = inParameters.zw;
  float2 viewportScale = 1.0 / (0.5 * viewportSize);
  float2 scaledSize = size * viewportScale;
  float2 center = viewportSize * (0.5 + 0.5 * p.xy);
  float aspect = viewportSize.x / viewportSize.y;
  float depth = alwaysOnTop ? (1.0 - COGS_CLEAR_DEPTH) : p.z;
  float2 scaledCenter = (center + offset) * viewportScale;
  float4 mask = float4(index == 0.f,
                   index == 1.f,
                   index == 2.f,
                   index == 3.f);
  texCoords = float2(dot(mask, inTexCoords.xzxz),
                     dot(mask, inTexCoords.yyww));
  weights = float4(dot(mask, float4(0, 1, 0, 1)),
                   dot(mask, float4(0, 0, 1, 1)),
                   0,
                   0);

#ifdef COGS_SPRITES_NOALIGNPIXEL
  float2 ll = center + offset + float2(-0.5, -0.5) * size;
#else
  float2 ll = floor(center + offset + float2(-0.5, -0.5) * size);
#endif

#ifdef COGS_SPRITES_IMAGETRANSFORM
  float2 v = scaledCenter + TransformImage((weights.xy-float2(0.5, 0.5)) * scaledSize, aspect) - float2(1, 1);
#else
  float2 v = (ll + weights.xy * size) * viewportScale - float2(1, 1);
#endif
  position = float4(v, depth, 1.0);
}
