#define COGS_COMBINE(a, b) a##b

vec3 GetNormal(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  vec3 normal = normalize(In.Normal);

#if COGS_STANDARD_MATERIAL_NORMAL_UV == 0
  vec3 normalColor = 2.0 * texture(normalMap, In.TexCoord0).rgb - vec3(1.0);
#elif COGS_STANDARD_MATERIAL_NORMAL_UV == 1
  vec3 normalColor = 2.0 * texture(normalMap, In.TexCoord1).rgb - vec3(1.0);
#endif

  vec3 tangent = normalize(In.Tangent.xyz);

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  vec3 bitangent = normalize(In.Bitangent);
#else
  vec3 bitangent = In.Tangent.w * normalize(cross(normal, tangent));
#endif

  vec3 bumpNormal = normalize(normalColor.z * normal + normalColor.x * tangent + normalColor.y * bitangent);
  normal = normalize(mix(normal, bumpNormal, normalMapFactor));
  
  return In.IsFrontFace ? normal : -normal;
#else
  return normalize(In.IsFrontFace ? In.Normal : -In.Normal);
#endif
}

vec4 GetAlbedo(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_ALBEDO_MAP

//  return albedo * texture(albedoMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_ALBEDO_UV).xy);
#if COGS_STANDARD_MATERIAL_ALBEDO_UV == 0
  return albedo * texture(albedoMap, In.TexCoord0.xy);
#elif COGS_STANDARD_MATERIAL_ALBEDO_UV == 1
  return albedo * texture(albedoMap, In.TexCoord1.xy);
#endif

#else
  return albedo;
#endif
}

float GetRoughness(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_ROUGHNESS_MAP

//  return roughness * texture(roughnessMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_ROUGHNESS_UV)).r;
#if COGS_STANDARD_MATERIAL_ROUGHNESS_UV == 0
  return roughness * texture(roughnessMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_ROUGHNESS_UV == 1
  return roughness * texture(roughnessMap, In.TexCoord1).r;
#endif

#elif defined(COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_MAP)

//  return roughness * texture(metallicRoughnessMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV)).g;
#if COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 0
  return roughness * texture(metallicRoughnessMap, In.TexCoord0).g;
#elif COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 1
  return roughness * texture(metallicRoughnessMap, In.TexCoord1).g;
#endif

#else
  return roughness;
#endif
}

float GetMetallic(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_METALLIC_MAP

//  return metallic * texture(metallicMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_METALLIC_UV)).r;
#if COGS_STANDARD_MATERIAL_METALLIC_UV == 0
  return metallic * texture(metallicMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_METALLIC_UV == 1
  return metallic * texture(metallicMap, In.TexCoord1).r;
#endif

#elif defined(COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_MAP)

//  return metallic * texture(metallicRoughnessMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV)).b;
#if COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 0
  return metallic * texture(metallicRoughnessMap, In.TexCoord0).b;
#elif COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV == 1
  return metallic * texture(metallicRoughnessMap, In.TexCoord1).b;
#endif

#else
  return metallic;
#endif
}

float GetGloss(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_GLOSS_MAP

//  return gloss * texture(glossMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_GLOSS_UV)).r;
#if COGS_STANDARD_MATERIAL_GLOSS_UV == 0
  return gloss * texture(glossMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_GLOSS_UV == 1
  return gloss * texture(glossMap, In.TexCoord1).r;
#endif

#elif defined(COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_MAP)

//  return gloss * texture(specularGlossMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV)).a;
#if COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 0
  return gloss * texture(specularGlossMap, In.TexCoord0).a;
#elif COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 1
  return gloss * texture(specularGlossMap, In.TexCoord1).a;
#endif

#else
  return gloss;
#endif
}

vec3 GetSpecular(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_SPECULAR_MAP

//  return specular * texture(specularMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_SPECULAR_UV)).rgb;
#if COGS_STANDARD_MATERIAL_SPECULAR_UV == 0
  return specular * texture(specularMap, In.TexCoord0).rgb;
#elif COGS_STANDARD_MATERIAL_SPECULAR_UV == 1
  return specular * texture(specularMap, In.TexCoord1).rgb;
#endif

#elif defined(COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_MAP)

//  return specular * texture(specularGlossMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV)).rgb;
#if COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 0
  return specular * texture(specularGlossMap, In.TexCoord0).rgb;
#elif COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV == 1
  return specular * texture(specularGlossMap, In.TexCoord1).rgb;
#endif

#else
  return specular;
#endif
}

vec3 GetEmissive(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_EMISSIVE_MAP

//  return emissive * texture(emissiveMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_EMISSIVE_UV).xy).rgb;
#if COGS_STANDARD_MATERIAL_EMISSIVE_UV == 0
  return emissive * texture(emissiveMap, In.TexCoord0.xy).rgb;
#elif COGS_STANDARD_MATERIAL_EMISSIVE_UV == 1
  return emissive * texture(emissiveMap, In.TexCoord1.xy).rgb;
#endif

#else
  return emissive;
#endif
}

float GetOcclusion(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_OCCLUSION_MAP

//  return occlusion * texture(occlusionMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_OCCLUSION_UV)).r;
#if COGS_STANDARD_MATERIAL_OCCLUSION_UV == 0
  return occlusion * texture(occlusionMap, In.TexCoord0).r;
#elif COGS_STANDARD_MATERIAL_OCCLUSION_UV == 1
  return occlusion * texture(occlusionMap, In.TexCoord1).r;
#endif

#else
  return occlusion;
#endif
}
