#include "Standard/StandardLighting.es30.glsl"
#include "Standard/StandardInput.es30.glsl"

SurfaceOut surfaceFunction(VertexIn In)
{
  SurfaceOut Out = createSurfaceOut();

  vec3 viewDir = normalize(eyePosition.xyz - In.WorldPosition.xyz);

  vec3 normal = GetNormal(In);
  vec4 albedo = GetAlbedo(In);
  float occlusion = GetOcclusion(In);
  vec3 emissive = GetEmissive(In);

#if defined(COGS_STANDARD_MATERIAL_ALBEDO_PER_VERTEX) || defined(COGS_STANDARD_MATERIAL_OUTPUT_ALBEDO_PER_VERTEX)
  albedo *= In.Albedo;
#endif

#ifdef COGS_STANDARD_MATERIAL_METALLIC_WORKFLOW
  float metallic = GetMetallic(In);
  float roughness = GetRoughness(In);

  float F0 = mix(0.04, 1.0, metallic);
#ifdef COGS_GLTF_FIXES
  vec3 dielectricSpecular = vec3(0.04, 0.04, 0.04);
  vec3 specularColor = mix(dielectricSpecular, albedo.rgb, metallic);
  vec3 diffuseColor = mix(albedo.rgb * (1.0 - dielectricSpecular.r), vec3(0), metallic);
#else
  vec3 specularColor = F0 * mix(vec3(F0, F0, F0), albedo.rgb, metallic);
  vec3 diffuseColor = (1.0 - F0) * mix(albedo.rgb, vec3(0), metallic);
   #endif
#else
  float roughness = 1.0 - GetGloss(In);
  vec3 specular = GetSpecular(In);

  float F0 = max(specular.r, max(specular.g, specular.b));

  vec3 specularColor = specular;
  vec3 diffuseColor = (1.0 - F0) * albedo.rgb;
#endif
#ifdef COGS_GLTF_FIXES
  roughness = sqrt(roughness);
#endif

  vec4 indirectLighting = vec4(0, 0, 0, 1);
  vec3 color = vec3(0, 0, 0);

  for (int i = 0; i < COGS_MAX_LIGHTS; ++i) {
    if (i >= numLights.x) break;

    vec3 lightDir;
    float attenuation;
    if(lightDirections[i].w == 0.0) {
        lightDir = -lightDirections[i].xyz;
        attenuation = 1.0;
    }
    else {
        lightDir = -(In.WorldPosition.xyz - lightPositions[i].xyz);
        float radius = lightParameters[i].a;
        float distance = length(lightDir);
        lightDir *= 1.0 / distance;
        attenuation = clamp(1.0 - (distance * distance) / (radius * radius), 0.0, 1.0);
        attenuation *= attenuation;
    }

    float shadowFactor = 1.0;

#ifdef COGS_FORWARD_SHADOWS_ENABLED
    if (lightDirections[i].w == .00) {
#if COGS_NUM_DIRECTIONAL_SHADOWS > 0
      shadowFactor = indirectLighting.a * calculateShadowContribution(In.WorldPosition, normal, lightDir, i, In.ScreenPos);
#endif
    } else {
#if COGS_NUM_POINT_SHADOWS > 0
      shadowFactor = indirectLighting.a * calculatePointLightShadow(lightPositions[i], In.WorldPosition.xyz, normal, i);
#endif
    }
#endif

    color += shadowFactor * StandardLighting(lightColorIntensity[i].rgb * lightColorIntensity[i].a * attenuation, normal, viewDir, lightDir, specularColor, diffuseColor, roughness, F0, occlusion);
  }

  color += StandardAmbient(normal, viewDir, specularColor, diffuseColor, roughness, F0, occlusion);

  Out.Normal = normal;
  Out.Color = vec4(color + emissive, albedo.a);

  return Out;
}
